package mainEngine;

import javafx.collections.ObservableList;
import dataModel.Dish;
import dataModel.Employee;
/**
* <h1>IMainEngine</h1>
* Interface which provides a contract to the clients
* regarding (a) the load of the input files, (b) the 
* computation of the statistics and (c) the creation
* of reports
*
* @version 1.0
* @since   2017-07-22
*/
public interface IMainEngine {
	
	/**
 	 * Loads the data from the input files
 	 * @return return 0 if executed properly
 	 */
	public int loadAllData();
	
	/**
 	 * Computes the weekly statistics and the statistics
 	 * for each employee and prints the results
 	 * 
 	 */
	public void computeAllStats();
	
	/**
 	 * Creates the different types of reports
 	 * @param mode a String to denote how to output reports. Valid values: "ReportGeneratorForFiles" / "ReportGeneratorForHTML"
 	 */
	public void createReports(String mode);
	
	public void getEmployees(ObservableList<Employee> data);
	public void getDishes(ObservableList<Dish> data);
}
